@@ -1,6 +1,6 @@
 import importlib.metadata
 
-from manufacturing.analysis import calc_ppk, suggest_control_limits
+from manufacturing.analysis import calc_ppk, suggest_specification_limits
 from manufacturing.data_import import import_csv, import_excel
 from manufacturing.report import generate_production_report
 from manufacturing.visual import (
@@ -8,6 +8,7 @@ from manufacturing.visual import (
     control_chart_base,
     control_plot,
     cpk_plot,
+    p_chart,
     ppk_plot,
     x_mr_chart,
     xbar_r_chart,
@@ -24,8 +25,9 @@ __all__ = [
     "generate_production_report",
     "import_csv",
     "import_excel",
+    "p_chart",
     "ppk_plot",
-    "suggest_control_limits",
+    "suggest_specification_limits",
     "x_mr_chart",
     "xbar_r_chart",
     "xbar_s_chart",
