@@ -11,7 +11,7 @@ _logger = logging.getLogger(__name__)
 
 
 def normality_test(
-    data: (List[int], List[float], pd.Series, np.array), alpha: float = 0.05
+    data: (List[int], List[float], pd.Series, np.ndarray), alpha: float = 0.05
 ):
     """
     Checks the data for normality and returns True if normality can't be demonstrated False.
@@ -23,7 +23,7 @@ def normality_test(
     _logger.debug("checking data for normality")
 
     stat, p = shapiro(data)
-    _logger.debug(f"shapiro statistics={stat:.03f}, p={p:.03f}")
+    _logger.debug(f"shapiro statistics={stat:.03g}, p={p:.03g}")
     if p > alpha:
         is_normal_shapiro_test = True
         _logger.debug("shapiro test indicates that the distribution is normal")
@@ -39,7 +39,7 @@ def normality_test(
         success = False
 
     if success:
-        _logger.debug(f"k^2 statistics={stat:.03f}, p={p:.03f}")
+        _logger.debug(f"k^2 statistics={stat:.03g}, p={p:.03g}")
         if p > alpha:
             is_normal_k2 = True
             _logger.debug("k^2 test indicates that the distribution is normal")
@@ -61,8 +61,8 @@ def normality_test(
     return is_normal
 
 
-def suggest_control_limits(
-    data: (List[int], List[float], pd.Series, np.array), sigma_level: float = 3.0
+def suggest_specification_limits(
+    data: (List[int], List[float], pd.Series, np.ndarray), sigma_level: float = 3.0
 ):
     """
     Given a data set and a sigma level, will return a dict containing the `upper_control_limit` and \
@@ -71,29 +71,29 @@ def suggest_control_limits(
     :param data: the data to be analyzed
     :param sigma_level: the sigma level; the default value is 3.0, but some users \
     may prefer a higher sigma level for their process
-    :return: a `dict` containing the `upper_control_limit` and `lower_control_limit` keys
+    :return: a ``tuple`` containing the ``upper_specification_limit`` and ``upper_specification_limit`` keys
     """
-    _logger.debug("defining control limits...")
+    _logger.debug("defining specification limits...")
     data = coerce(data)
     normality_test(data)
 
     mean = data.mean()
-    sigma = data.std()
+    sigma = data.std() * 1.25  # add a buffer to allow normal process variation
 
     return mean - sigma_level * sigma, mean + sigma_level * sigma
 
 
 def calc_pp(
-    data: (List[int], List[float], pd.Series, np.array),
-    upper_control_limit: (int, float),
-    lower_control_limit: (int, float),
+    data: (List[int], List[float], pd.Series, np.ndarray),
+    upper_specification_limit: (int, float),
+    lower_specification_limit: (int, float),
 ):
     """
     Calculate and return the Pp of the provided dataset given the control limits.
 
     :param data: the data to be analyzed
-    :param upper_control_limit: the upper control limit
-    :param lower_control_limit: the lower control limit
+    :param upper_specification_limit: the upper control limit
+    :param lower_specification_limit: the lower control limit
     :return: the pp level
     """
     _logger.debug("calculating pp...")
@@ -101,23 +101,23 @@ def calc_pp(
 
     normality_test(data)
 
-    pp = (upper_control_limit - lower_control_limit) / 6 * data.std()
+    pp = (upper_specification_limit - lower_specification_limit) / 6 * data.std()
 
-    _logger.debug(f"cp = {pp:.03f} on the supplied dataset of length {len(data)}")
+    _logger.debug(f"cp = {pp:.03g} on the supplied dataset of length {len(data)}")
 
     return pp
 
 
 def calc_ppu(
-    data: (List[int], List[float], pd.Series, np.array),
-    upper_control_limit: (int, float),
+    data: (List[int], List[float], pd.Series, np.ndarray),
+    upper_specification_limit: (int, float),
     skip_normality_test: bool = True,
 ):
     """
     Calculate and return the Pp (upper) of the provided dataset given the upper control limit.
 
     :param data: the data to be analyzed
-    :param upper_control_limit: the upper control limit
+    :param upper_specification_limit: the upper control limit
     :param skip_normality_test: used when the normality test is not necessary
     :return: the pp level
     """
@@ -130,7 +130,7 @@ def calc_ppu(
     mean = data.mean()
     std_dev = data.std()
 
-    ppu = (upper_control_limit - mean) / (3 * std_dev)
+    ppu = (upper_specification_limit - mean) / (3 * std_dev)
 
     _logger.debug(
         f"dataset of length {len(data)}, " f"mean={mean}, " f"std_dev={std_dev}"
@@ -141,15 +141,15 @@ def calc_ppu(
 
 
 def calc_ppl(
-    data: (List[int], List[float], pd.Series, np.array),
-    lower_control_limit: (int, float),
+    data: (List[int], List[float], pd.Series, np.ndarray),
+    lower_specification_limit: (int, float),
     skip_normality_test=True,
 ):
     """
     Calculate and return the Pp (lower) of the provided dataset given the lower control limit.
 
     :param data: the data to be analyzed
-    :param lower_control_limit: the lower control limit
+    :param lower_specification_limit: the lower control limit
     :param skip_normality_test: used when the normality test is not necessary
     :return: the pp level
     """
@@ -162,7 +162,7 @@ def calc_ppl(
     mean = data.mean()
     std_dev = data.std()
 
-    ppl = (mean - lower_control_limit) / (3 * std_dev)
+    ppl = (mean - lower_specification_limit) / (3 * std_dev)
 
     _logger.debug(
         f"dataset of length {len(data)}, " f"mean={mean}, " f"std_dev={std_dev}"
@@ -173,16 +173,16 @@ def calc_ppl(
 
 
 def calc_ppk(
-    data: (List[int], List[float], pd.Series, np.array),
-    upper_control_limit: (int, float),
-    lower_control_limit: (int, float),
+    data: (List[int], List[float], pd.Series, np.ndarray),
+    upper_specification_limit: (int, float),
+    lower_specification_limit: (int, float),
 ):
     """
     Calculate and return the Pp (upper) of the provided dataset given the upper control limit.
 
     :param data: the data to be analyzed
-    :param upper_control_limit: the upper control limit
-    :param lower_control_limit: the lower control limit
+    :param upper_specification_limit: the upper specification limit
+    :param lower_specification_limit: the lower control limit
     :return: the ppk level
     """
     _logger.debug("calculating ppk...")
@@ -192,12 +192,16 @@ def calc_ppk(
 
     zupper = abs(
         calc_ppu(
-            data=data, upper_control_limit=upper_control_limit, skip_normality_test=True
+            data=data,
+            upper_specification_limit=upper_specification_limit,
+            skip_normality_test=True,
         )
     )
     zlower = abs(
         calc_ppl(
-            data=data, lower_control_limit=lower_control_limit, skip_normality_test=True
+            data=data,
+            lower_specification_limit=lower_specification_limit,
+            skip_normality_test=True,
         )
     )
 
@@ -205,10 +209,10 @@ def calc_ppk(
 
     _logger.debug(
         f"dataset of length {len(data)}, "
-        f"zupper={zupper:.03f}, "
-        f"zlower={zlower:.03f}"
+        f"zupper={zupper:.03g}, "
+        f"zlower={zlower:.03g}"
     )
-    _logger.debug(f"cpk = {cpk:.03f}")
+    _logger.debug(f"cpk = {cpk:.03g}")
 
     ratio = zupper / zlower
     if ratio < 1:
@@ -224,7 +228,7 @@ def calc_ppk(
 
 
 def control_beyond_limits(
-    data: (List[int], List[float], pd.Series, np.array),
+    data: (List[int], List[float], pd.Series, np.ndarray),
     upper_control_limit: (int, float),
     lower_control_limit: (int, float),
 ) -> pd.Series:
@@ -249,7 +253,7 @@ def control_beyond_limits(
 
 
 def control_zone_a(
-    data: (List[int], List[float], pd.Series, np.array),
+    data: (List[int], List[float], pd.Series, np.ndarray),
     upper_control_limit: (int, float),
     lower_control_limit: (int, float),
 ) -> pd.Series:
@@ -274,12 +278,34 @@ def control_zone_a(
     for i in range(len(data) - 2):
         points = data[i : i + 3].to_numpy()
 
-        count = 0
-        for point in points:
-            if point < zone_b_lower_limit or point > zone_b_upper_limit:
-                count += 1
+        try:
+            iter(zone_b_lower_limit)
+            zblls = zone_b_lower_limit[i : i+3]
+        except TypeError:
+            zblls = None
 
-        if count >= 2:
+        try:
+            iter(zone_b_upper_limit)
+            zbuls = zone_b_upper_limit[i : i+3]
+        except TypeError:
+            zbuls = None
+
+        if zblls is not None:
+            values = [1 for p in points if p < zblls[i]]
+        else:
+            values = [1 for p in points if p < zone_b_lower_limit]
+
+        if sum(values) > 2:
+            index = i + np.arange(3)
+            violations.append(pd.Series(data=points, index=index))
+            _logger.info(f"zone a violation found at index {i}")
+
+        if zbuls is not None:
+            values = [1 for p in points if p > zone_b_upper_limit[i]]
+        else:
+            values = [1 for p in points if p > zone_b_upper_limit]
+
+        if sum(values) > 2:
             index = i + np.arange(3)
             violations.append(pd.Series(data=points, index=index))
             _logger.info(f"zone a violation found at index {i}")
@@ -293,7 +319,7 @@ def control_zone_a(
 
 
 def control_zone_b(
-    data: (List[int], List[float], pd.Series, np.array),
+    data: (List[int], List[float], pd.Series, np.ndarray),
     upper_control_limit: (int, float),
     lower_control_limit: (int, float),
 ) -> pd.Series:
@@ -315,16 +341,36 @@ def control_zone_b(
 
     # looking for violations in which 2 out of 3 are in zone A or beyond
     violations = []
-    for i in range(len(data) - 5):
+    for i in range(len(data) - 4):
         points = data[i : i + 5].to_numpy()
+        try:
+            iter(zone_c_lower_limit)
+            zclls = zone_c_lower_limit[i : i+5]
+        except TypeError:
+            zclls = None
 
-        count = 0
-        for point in points:
-            if point < zone_c_lower_limit or point > zone_c_upper_limit:
-                count += 1
+        try:
+            iter(zone_c_upper_limit)
+            zculs = zone_c_upper_limit[i : i+5]
+        except TypeError:
+            zculs = None
 
-        if count >= 4:
-            index = i + np.arange(5)
+        if zclls is not None:
+            values = [1 for p in points if p < zclls[i]]
+        else:
+            values = [1 for p in points if p < zone_c_lower_limit]
+
+        if sum(values) > 3:
+            index = i + np.arange(5) + data.index[0]
+            violations.append(pd.Series(data=points, index=index))
+            _logger.info(f"zone b violation found at index {i}")
+
+        if zculs is not None:
+            values = [1 for p in points if p > zculs[i]]
+        else:
+            values = [1 for p in points if p > zone_c_upper_limit]
+        if sum(values) > 3:
+            index = i + np.arange(5) + data.index[0]
             violations.append(pd.Series(data=points, index=index))
             _logger.info(f"zone b violation found at index {i}")
 
@@ -333,11 +379,12 @@ def control_zone_b(
 
     s = pd.concat(violations)
     s = s.loc[~s.index.duplicated()]
+
     return s
 
 
 def control_zone_c(
-    data: (List[int], List[float], pd.Series, np.array),
+    data: (List[int], List[float], pd.Series, np.ndarray),
     upper_control_limit: (int, float),
     lower_control_limit: (int, float),
 ):
@@ -359,23 +406,19 @@ def control_zone_c(
     violations = []
     for i in range(len(data) - 6):
         points = data[i : i + 7].to_numpy()
+        try:
+            iter(spec_center)
+            center_points = spec_center[i:i+7]
+        except TypeError:
+            center_points = None
 
-        count = 1
-        above = data[i] > spec_center
-        for point in points[1:]:
-            if above is True:
-                if point > spec_center:
-                    count += 1
-                else:
-                    break
-            else:
-                if point < spec_center:
-                    count += 1
-                else:
-                    break
+        if center_points is not None:
+            values = [1 if p > c else -1 for p, c in zip(points, center_points)]
+        else:
+            values = [1 if p > spec_center else -1 for p in points]
 
-        if count >= 7:
-            index = i + np.arange(7)
+        if abs(sum(values)) > 6:
+            index = i + np.arange(7) + data.index[0]
             violations.append(pd.Series(data=points, index=index))
             _logger.info(f"zone c violation found at index {i}")
 
@@ -388,8 +431,13 @@ def control_zone_c(
 
 
 def control_zone_trend(
-    data: (List[int], List[float], pd.Series, np.array)
+    data: (List[int], List[float], pd.Series, np.ndarray)
 ) -> pd.Series:
+    """
+    Returns a pandas.Series containing the data in which 6 consecutive points are in the same direction
+    :param data:
+    :return:
+    """
     _logger.debug("identifying control trend violations...")
     data = coerce(data)
 
@@ -407,8 +455,10 @@ def control_zone_trend(
         neg_dataset = [v for v in diff_data[i : i + 7] if v <= 0]
 
         if len(pos_dataset) >= 7 or len(neg_dataset) >= 7:
+            points = data[i : i + 7].to_numpy()
+            index = i + np.arange(7) + data.index[0]
             try:
-                violations.append(pd.Series(index=[i + 6], data=data[i + 6]))
+                violations.append(pd.Series(index=index, data=points))
                 _logger.info(f"trend violation found at index {i+6}")
             except KeyError:
                 break
@@ -422,7 +472,7 @@ def control_zone_trend(
 
 
 def control_zone_mixture(
-    data: (List[int], List[float], pd.Series, np.array),
+    data: (List[int], List[float], pd.Series, np.ndarray),
     upper_control_limit: (int, float),
     lower_control_limit: (int, float),
 ) -> pd.Series:
@@ -442,19 +492,15 @@ def control_zone_mixture(
     zone_c_upper_limit = spec_center + spec_range / 3
     zone_c_lower_limit = spec_center - spec_range / 3
 
-    # looking for violations in which 2 out of 3 are in zone A or beyond
+    # looking for violations in which 8 points occur with none in zone C
     violations = []
     for i in range(len(data) - 7):
         points = data[i : i + 8].to_numpy()
+        values = [
+            1 for p in points if p > zone_c_upper_limit and p < zone_c_lower_limit
+        ]
 
-        count = 0
-        for point in points:
-            if not zone_c_lower_limit < point < zone_c_upper_limit:
-                count += 1
-            else:
-                break
-
-        if count >= 8:
+        if sum(values) > 7:
             index = i + np.arange(8)
             violations.append(pd.Series(data=points, index=index))
             _logger.info(f"mixture violation found at index {i}")
@@ -468,7 +514,7 @@ def control_zone_mixture(
 
 
 def control_zone_stratification(
-    data: (List[int], List[float], pd.Series, np.array),
+    data: (List[int], List[float], pd.Series, np.ndarray),
     upper_control_limit: (int, float),
     lower_control_limit: (int, float),
 ) -> pd.Series:
@@ -492,12 +538,10 @@ def control_zone_stratification(
     violations = []
     for i in range(len(data) - 14):
         points = data[i : i + 15].to_numpy()
-
-        points = points[
-            np.logical_and(points < zone_c_upper_limit, points > zone_c_lower_limit)
+        values = [
+            1 if zone_c_lower_limit < p < zone_c_upper_limit else 0 for p in points
         ]
-
-        if len(points) >= 15:
+        if sum(values) > 14:
             index = i + np.arange(15)
             violations.append(pd.Series(data=points, index=index))
             _logger.info(f"stratification violation found at index {i}")
@@ -507,23 +551,24 @@ def control_zone_stratification(
 
     s = pd.concat(violations)
     s = s.loc[~s.index.duplicated()]
+
     return s
 
 
 def control_zone_overcontrol(
-    data: (List[int], List[float], pd.Series, np.array),
+    data: (List[int], List[float], pd.Series, np.ndarray),
     upper_control_limit: (int, float),
     lower_control_limit: (int, float),
 ):
     """
-    Returns a pandas.Series containing the data in which 14 consecutive points are alternating above/below the center.
+    Returns a pandas.Series containing the data in which 14 consecutive points are alternating in direction.
 
     :param data: The data to be analyzed
     :param upper_control_limit: the upper control limit
     :param lower_control_limit: the lower control limit
     :return: a pandas.Series object with all out-of-control points
     """
-    _logger.debug("identifying control over-control violations...")
+    _logger.debug("identifying over-control violations...")
     data = coerce(data)
 
     spec_range = (upper_control_limit - lower_control_limit) / 2
@@ -533,20 +578,26 @@ def control_zone_overcontrol(
     violations = []
     for i in range(len(data) - 14):
         points = data[i : i + 14].to_numpy()
-        odds = points[::2]
-        evens = points[1::2]
+        diffs = np.diff(points)
 
-        if odds[0] > 0.0:
-            odds = odds[odds > spec_center]
-            evens = evens[evens < spec_center]
-        else:
-            odds = odds[odds < spec_center]
-            evens = evens[evens > spec_center]
+        values = [1 if d > 0 else d for d in diffs]
+        values = [-1 if v < 0 else v for v in values]
 
-        if len(odds) == len(evens) == 7:
+        # count through to look for overcontrol
+        last_value = -values[0]
+        count = 0
+        for v in values:
+            if v != last_value:
+                count += 1
+            else:
+                count = 0
+                break
+            last_value = v
+
+        if count >= 13:
             index = i + np.arange(14)
             violations.append(pd.Series(data=points, index=index))
-            _logger.info(f"over-control violation found at index {i}")
+            _logger.info(f"overcontrol violation found at index {i}")
 
     if len(violations) == 0:
         return pd.Series(dtype="float64")
